@@ -3,5 +3,8 @@ if(getRversion() >= "2.10") {
                            "covariates", "isCBPS.r", "isGlm.m", "isGlm.y", "isLm.m",
                            "isMultiConfounders", "isNominal.m", "isOrdinal.m",
                            "isSumStat.r", "n.y", "num.ms", "outcome", "pfun", "phi1",
-                           "treat", "w", "y.data"))
+                           "treat", "w", "y.data",
+                           "model.weights", "par", "contour", "points", "text",
+                           "relevel", "vcov", "X", "cov", "var", "qnorm",
+                           "weights"))
 }
\ No newline at end of file
